# Aplikacja korzysta z platformy Dash: dash.plotly.com/
# Aby uruchomić aplikację:
# 1. Instalujemy potrzebne biblioteki (wg listy importowanych niżej lub za pomocą polecenia pip install -r requirements.txt)
# UWAGA: zainstalowanie biblioteki torch tak, aby korzystała z procesora graficznego,
# może wymagać specjalnego polecenia zgodnie z instrukcjami tutaj: https://pytorch.org/get-started/locally/
# 2. Uruchamiamy polecenie python3 clip.py
# 3. Przechodzimy w przeglądarce pod adres localhost:8050
# Aby zatrzymać aplikację: wracamy do terminala i używamy kombinacji klawiszy CTRL+C

import torch
import requests
from dash import Dash, dcc, callback, Output, Input
import dash_bootstrap_components as dbc
from transformers import AutoModel, AutoProcessor
from PIL import Image

device = "cuda" if torch.cuda.is_available() else "cpu"
model_path = "openai/clip-vit-large-patch14"
model = AutoModel.from_pretrained(model_path).to(device)
processor = AutoProcessor.from_pretrained(model_path)
img_url = "https://upload.wikimedia.org/wikipedia/en/1/1f/WomanYellingAtACat_meme.jpg"
img = Image.open(requests.get(img_url, stream = True).raw)

app = Dash(__name__, external_stylesheets=[dbc.themes.BOOTSTRAP])

app.layout = dbc.Container([
    dbc.Row([
        dbc.Col(dbc.Row([dcc.Input(id = "image_url", type = "text", value = "https://upload.wikimedia.org/wikipedia/en/1/1f/WomanYellingAtACat_meme.jpg")]), width = 8),
        dbc.Col([
            dbc.Row([dcc.Input(id = "label_1", type = "text", value = "cat")]),
            dbc.Row([dcc.Input(id = "label_2", type = "text", value = "dog")]),
        ], width = 4),
    ]),
    dbc.Row([
        dbc.Card([
            dbc.CardImg(id = "image_preview", src = img)
        ])
    ]),
    dbc.Row([
        dcc.Markdown(id = "classification", children = None )
    ])
    
])


@callback(
    Output('image_preview', 'src'),
    Output('classification', 'children'),
    Input('image_url', 'value'),
    Input('label_1', 'value'),
    Input('label_2', 'value')
)
def update_image(image_url, label_1, label_2):
    img = Image.open(requests.get(image_url, stream = True).raw)
    image_input = processor(images = img,  return_tensors="pt").to(device)
    tag_list = [label_1, label_2]
    text_inputs = processor(text = tag_list, padding = True, return_tensors = "pt").to(device)

    with torch.no_grad():
        image_features = model.get_image_features(**image_input)
        text_features = model.get_text_features(**text_inputs)
    image_features /= image_features.norm(dim=-1, keepdim=True)
    text_features /= text_features.norm(dim=-1, keepdim=True)
    similarity = (10* image_features @ text_features.T).softmax(dim=-1)
    values, indices = similarity[0].topk(2)
    tags = [f"{tag_list[index]:>16s}: {100 * value.item():.2f}%" for value, index in zip(values, indices)]
    tags = tags[0] + "   \n" + tags[1] 
    

    
    return img, tags


if __name__ == '__main__':
    app.run_server(debug=True)